@@ -25,6 +25,7 @@
  *   2021      Zhi An Ng <zhin@google.com> (Copyright owned by Google, LLC)
  *   2021      Décio Luiz Gazzoni Filho <decio@decpp.net>
  *   2023      Yi-Yen Chung <eric681@andestech.com> (Copyright owned by Andes Technology)
+ *   2023      Chi-Wei Chu <wewe5215@gapp.nthu.edu.tw> (Copyright owned by NTHU pllab)
  */
 
 #if !defined(SIMDE_ARM_NEON_LD1Q_X4_H)
@@ -51,9 +52,16 @@ simde_vld1q_f16_x4(simde_float16_t const ptr[HEDLEY_ARRAY_PARAM(32)]) {
     return vld1q_f16_x4(ptr);
   #else
     simde_float16x8_private a_[4];
-    for (size_t i = 0; i < 32; i++) {
-      a_[i / 8].values[i % 8] = ptr[i];
-    }
+    #if defined(SIMDE_RISCV_V_NATIVE) && SIMDE_ARCH_RISCV_ZVFH
+      a_[0].sv128 = __riscv_vle16_v_f16m1((_Float16 *)ptr , 8);
+      a_[1].sv128 = __riscv_vle16_v_f16m1((_Float16 *)(ptr+8) , 8);
+      a_[2].sv128 = __riscv_vle16_v_f16m1((_Float16 *)(ptr+16) , 8);
+      a_[3].sv128 = __riscv_vle16_v_f16m1((_Float16 *)(ptr+24) , 8);
+    #else
+      for (size_t i = 0; i < 32; i++) {
+        a_[i / 8].values[i % 8] = ptr[i];
+      }
+    #endif
     simde_float16x8x4_t s_ = { { simde_float16x8_from_private(a_[0]),
                                  simde_float16x8_from_private(a_[1]),
                                  simde_float16x8_from_private(a_[2]),
@@ -76,9 +84,16 @@ simde_vld1q_f32_x4(simde_float32 const ptr[HEDLEY_ARRAY_PARAM(16)]) {
     return vld1q_f32_x4(ptr);
   #else
     simde_float32x4_private a_[4];
-    for (size_t i = 0; i < 16; i++) {
-      a_[i / 4].values[i % 4] = ptr[i];
-    }
+    #if defined(SIMDE_RISCV_V_NATIVE)
+      a_[0].sv128 = __riscv_vle32_v_f32m1(ptr , 4);
+      a_[1].sv128 = __riscv_vle32_v_f32m1(ptr+4 , 4);
+      a_[2].sv128 = __riscv_vle32_v_f32m1(ptr+8 , 4);
+      a_[3].sv128 = __riscv_vle32_v_f32m1(ptr+12 , 4);
+    #else
+      for (size_t i = 0; i < 16; i++) {
+        a_[i / 4].values[i % 4] = ptr[i];
+      }
+    #endif
     simde_float32x4x4_t s_ = { { simde_float32x4_from_private(a_[0]),
                                  simde_float32x4_from_private(a_[1]),
                                  simde_float32x4_from_private(a_[2]),
@@ -101,9 +116,16 @@ simde_vld1q_f64_x4(simde_float64 const ptr[HEDLEY_ARRAY_PARAM(8)]) {
     return vld1q_f64_x4(ptr);
   #else
     simde_float64x2_private a_[4];
-    for (size_t i = 0; i < 8; i++) {
-      a_[i / 2].values[i % 2] = ptr[i];
-    }
+    #if defined(SIMDE_RISCV_V_NATIVE)
+      a_[0].sv128 = __riscv_vle64_v_f64m1(ptr , 2);
+      a_[1].sv128 = __riscv_vle64_v_f64m1(ptr+2 , 2);
+      a_[2].sv128 = __riscv_vle64_v_f64m1(ptr+4 , 2);
+      a_[3].sv128 = __riscv_vle64_v_f64m1(ptr+6 , 2);
+    #else
+      for (size_t i = 0; i < 8; i++) {
+        a_[i / 2].values[i % 2] = ptr[i];
+      }
+    #endif
     simde_float64x2x4_t s_ = { { simde_float64x2_from_private(a_[0]),
                                  simde_float64x2_from_private(a_[1]),
                                  simde_float64x2_from_private(a_[2]),
@@ -126,9 +148,16 @@ simde_vld1q_s8_x4(int8_t const ptr[HEDLEY_ARRAY_PARAM(64)]) {
     return vld1q_s8_x4(ptr);
   #else
     simde_int8x16_private a_[4];
-    for (size_t i = 0; i < 64; i++) {
-      a_[i / 16].values[i % 16] = ptr[i];
-    }
+    #if defined(SIMDE_RISCV_V_NATIVE)
+      a_[0].sv128 = __riscv_vle8_v_i8m1(ptr , 16);
+      a_[1].sv128 = __riscv_vle8_v_i8m1(ptr+16 , 16);
+      a_[2].sv128 = __riscv_vle8_v_i8m1(ptr+32 , 16);
+      a_[3].sv128 = __riscv_vle8_v_i8m1(ptr+48 , 16);
+    #else
+      for (size_t i = 0; i < 64; i++) {
+        a_[i / 16].values[i % 16] = ptr[i];
+      }
+    #endif
     simde_int8x16x4_t s_ = { { simde_int8x16_from_private(a_[0]),
                                simde_int8x16_from_private(a_[1]),
                                simde_int8x16_from_private(a_[2]),
@@ -151,9 +180,16 @@ simde_vld1q_s16_x4(int16_t const ptr[HEDLEY_ARRAY_PARAM(32)]) {
     return vld1q_s16_x4(ptr);
   #else
     simde_int16x8_private a_[4];
-    for (size_t i = 0; i < 32; i++) {
-      a_[i / 8].values[i % 8] = ptr[i];
-    }
+    #if defined(SIMDE_RISCV_V_NATIVE)
+      a_[0].sv128 = __riscv_vle16_v_i16m1(ptr , 8);
+      a_[1].sv128 = __riscv_vle16_v_i16m1(ptr+8 , 8);
+      a_[2].sv128 = __riscv_vle16_v_i16m1(ptr+16 , 8);
+      a_[3].sv128 = __riscv_vle16_v_i16m1(ptr+24 , 8);
+    #else
+      for (size_t i = 0; i < 32; i++) {
+        a_[i / 8].values[i % 8] = ptr[i];
+      }
+    #endif
     simde_int16x8x4_t s_ = { { simde_int16x8_from_private(a_[0]),
                                simde_int16x8_from_private(a_[1]),
                                simde_int16x8_from_private(a_[2]),
@@ -176,9 +212,16 @@ simde_vld1q_s32_x4(int32_t const ptr[HEDLEY_ARRAY_PARAM(16)]) {
     return vld1q_s32_x4(ptr);
   #else
     simde_int32x4_private a_[4];
-    for (size_t i = 0; i < 16; i++) {
-      a_[i / 4].values[i % 4] = ptr[i];
-    }
+    #if defined(SIMDE_RISCV_V_NATIVE)
+      a_[0].sv128 = __riscv_vle32_v_i32m1(ptr , 4);
+      a_[1].sv128 = __riscv_vle32_v_i32m1(ptr+4 , 4);
+      a_[2].sv128 = __riscv_vle32_v_i32m1(ptr+8 , 4);
+      a_[3].sv128 = __riscv_vle32_v_i32m1(ptr+12 , 4);
+    #else
+      for (size_t i = 0; i < 16; i++) {
+        a_[i / 4].values[i % 4] = ptr[i];
+      }
+    #endif
     simde_int32x4x4_t s_ = { { simde_int32x4_from_private(a_[0]),
                                simde_int32x4_from_private(a_[1]),
                                simde_int32x4_from_private(a_[2]),
@@ -201,9 +244,16 @@ simde_vld1q_s64_x4(int64_t const ptr[HEDLEY_ARRAY_PARAM(8)]) {
     return vld1q_s64_x4(ptr);
   #else
     simde_int64x2_private a_[4];
-    for (size_t i = 0; i < 8; i++) {
-      a_[i / 2].values[i % 2] = ptr[i];
-    }
+    #if defined(SIMDE_RISCV_V_NATIVE)
+      a_[0].sv128 = __riscv_vle64_v_i64m1(ptr , 2);
+      a_[1].sv128 = __riscv_vle64_v_i64m1(ptr+2 , 2);
+      a_[2].sv128 = __riscv_vle64_v_i64m1(ptr+4 , 2);
+      a_[3].sv128 = __riscv_vle64_v_i64m1(ptr+6 , 2);
+    #else
+      for (size_t i = 0; i < 8; i++) {
+        a_[i / 2].values[i % 2] = ptr[i];
+      }
+    #endif
     simde_int64x2x4_t s_ = { { simde_int64x2_from_private(a_[0]),
                                simde_int64x2_from_private(a_[1]),
                                simde_int64x2_from_private(a_[1]),
@@ -226,9 +276,16 @@ simde_vld1q_u8_x4(uint8_t const ptr[HEDLEY_ARRAY_PARAM(64)]) {
     return vld1q_u8_x4(ptr);
   #else
     simde_uint8x16_private a_[4];
-    for (size_t i = 0; i < 64; i++) {
-      a_[i / 16].values[i % 16] = ptr[i];
-    }
+    #if defined(SIMDE_RISCV_V_NATIVE)
+      a_[0].sv128 = __riscv_vle8_v_u8m1(ptr , 16);
+      a_[1].sv128 = __riscv_vle8_v_u8m1(ptr+16 , 16);
+      a_[2].sv128 = __riscv_vle8_v_u8m1(ptr+32 , 16);
+      a_[3].sv128 = __riscv_vle8_v_u8m1(ptr+48 , 16);
+    #else
+      for (size_t i = 0; i < 64; i++) {
+        a_[i / 16].values[i % 16] = ptr[i];
+      }
+    #endif
     simde_uint8x16x4_t s_ = { { simde_uint8x16_from_private(a_[0]),
                                 simde_uint8x16_from_private(a_[1]),
                                 simde_uint8x16_from_private(a_[2]),
@@ -251,9 +308,16 @@ simde_vld1q_u16_x4(uint16_t const ptr[HEDLEY_ARRAY_PARAM(32)]) {
     return vld1q_u16_x4(ptr);
   #else
     simde_uint16x8_private a_[4];
-    for (size_t i = 0; i < 32; i++) {
-      a_[i / 8].values[i % 8] = ptr[i];
-    }
+    #if defined(SIMDE_RISCV_V_NATIVE)
+      a_[0].sv128 = __riscv_vle16_v_u16m1(ptr , 8);
+      a_[1].sv128 = __riscv_vle16_v_u16m1(ptr+8 , 8);
+      a_[2].sv128 = __riscv_vle16_v_u16m1(ptr+16 , 8);
+      a_[3].sv128 = __riscv_vle16_v_u16m1(ptr+24 , 8);
+    #else
+      for (size_t i = 0; i < 32; i++) {
+        a_[i / 8].values[i % 8] = ptr[i];
+      }
+    #endif
     simde_uint16x8x4_t s_ = { { simde_uint16x8_from_private(a_[0]),
                                 simde_uint16x8_from_private(a_[1]),
                                 simde_uint16x8_from_private(a_[2]),
@@ -276,9 +340,16 @@ simde_vld1q_u32_x4(uint32_t const ptr[HEDLEY_ARRAY_PARAM(16)]) {
     return vld1q_u32_x4(ptr);
   #else
     simde_uint32x4_private a_[4];
-    for (size_t i = 0; i < 16; i++) {
-      a_[i / 4].values[i % 4] = ptr[i];
-    }
+    #if defined(SIMDE_RISCV_V_NATIVE)
+      a_[0].sv128 = __riscv_vle32_v_u32m1(ptr , 4);
+      a_[1].sv128 = __riscv_vle32_v_u32m1(ptr+4 , 4);
+      a_[2].sv128 = __riscv_vle32_v_u32m1(ptr+8 , 4);
+      a_[3].sv128 = __riscv_vle32_v_u32m1(ptr+12 , 4);
+    #else
+      for (size_t i = 0; i < 16; i++) {
+        a_[i / 4].values[i % 4] = ptr[i];
+      }
+    #endif
     simde_uint32x4x4_t s_ = { { simde_uint32x4_from_private(a_[0]),
                                 simde_uint32x4_from_private(a_[1]),
                                 simde_uint32x4_from_private(a_[2]),
@@ -301,9 +372,16 @@ simde_vld1q_u64_x4(uint64_t const ptr[HEDLEY_ARRAY_PARAM(8)]) {
     return vld1q_u64_x4(ptr);
   #else
     simde_uint64x2_private a_[4];
-    for (size_t i = 0; i < 8; i++) {
-      a_[i / 2].values[i % 2] = ptr[i];
-    }
+    #if defined(SIMDE_RISCV_V_NATIVE)
+      a_[0].sv128 = __riscv_vle64_v_u64m1(ptr , 2);
+      a_[1].sv128 = __riscv_vle64_v_u64m1(ptr+2 , 2);
+      a_[2].sv128 = __riscv_vle64_v_u64m1(ptr+4 , 2);
+      a_[3].sv128 = __riscv_vle64_v_u64m1(ptr+6 , 2);
+    #else
+      for (size_t i = 0; i < 8; i++) {
+        a_[i / 2].values[i % 2] = ptr[i];
+      }
+    #endif
     simde_uint64x2x4_t s_ = { { simde_uint64x2_from_private(a_[0]),
                                 simde_uint64x2_from_private(a_[1]),
                                 simde_uint64x2_from_private(a_[2]),
@@ -325,9 +403,16 @@ simde_vld1q_p8_x4(simde_poly8_t const ptr[HEDLEY_ARRAY_PARAM(64)]) {
     return vld1q_p8_x4(ptr);
   #else
     simde_poly8x16_private a_[4];
-    for (size_t i = 0; i < 64; i++) {
-      a_[i / 16].values[i % 16] = ptr[i];
-    }
+    #if defined(SIMDE_RISCV_V_NATIVE)
+      a_[0].sv128 = __riscv_vle8_v_u8m1(ptr , 16);
+      a_[1].sv128 = __riscv_vle8_v_u8m1(ptr+16 , 16);
+      a_[2].sv128 = __riscv_vle8_v_u8m1(ptr+32 , 16);
+      a_[3].sv128 = __riscv_vle8_v_u8m1(ptr+48 , 16);
+    #else
+      for (size_t i = 0; i < 64; i++) {
+        a_[i / 16].values[i % 16] = ptr[i];
+      }
+    #endif
     simde_poly8x16x4_t s_ = { { simde_poly8x16_from_private(a_[0]),
                                 simde_poly8x16_from_private(a_[1]),
                                 simde_poly8x16_from_private(a_[2]),
@@ -349,9 +434,16 @@ simde_vld1q_p16_x4(simde_poly16_t const ptr[HEDLEY_ARRAY_PARAM(32)]) {
     return vld1q_p16_x4(ptr);
   #else
     simde_poly16x8_private a_[4];
-    for (size_t i = 0; i < 32; i++) {
-      a_[i / 8].values[i % 8] = ptr[i];
-    }
+    #if defined(SIMDE_RISCV_V_NATIVE)
+      a_[0].sv128 = __riscv_vle16_v_u16m1(ptr , 8);
+      a_[1].sv128 = __riscv_vle16_v_u16m1(ptr+8 , 8);
+      a_[2].sv128 = __riscv_vle16_v_u16m1(ptr+16 , 8);
+      a_[3].sv128 = __riscv_vle16_v_u16m1(ptr+24 , 8);
+    #else
+      for (size_t i = 0; i < 32; i++) {
+        a_[i / 8].values[i % 8] = ptr[i];
+      }
+    #endif
     simde_poly16x8x4_t s_ = { { simde_poly16x8_from_private(a_[0]),
                                 simde_poly16x8_from_private(a_[1]),
                                 simde_poly16x8_from_private(a_[2]),
@@ -373,9 +465,16 @@ simde_vld1q_p64_x4(simde_poly64_t const ptr[HEDLEY_ARRAY_PARAM(8)]) {
     return vld1q_p64_x4(ptr);
   #else
     simde_poly64x2_private a_[4];
-    for (size_t i = 0; i < 8; i++) {
-      a_[i / 2].values[i % 2] = ptr[i];
-    }
+    #if defined(SIMDE_RISCV_V_NATIVE)
+      a_[0].sv128 = __riscv_vle64_v_u64m1(ptr , 2);
+      a_[1].sv128 = __riscv_vle64_v_u64m1(ptr+2 , 2);
+      a_[2].sv128 = __riscv_vle64_v_u64m1(ptr+4 , 2);
+      a_[3].sv128 = __riscv_vle64_v_u64m1(ptr+6 , 2);
+    #else
+      for (size_t i = 0; i < 8; i++) {
+        a_[i / 2].values[i % 2] = ptr[i];
+      }
+    #endif
     simde_poly64x2x4_t s_ = { { simde_poly64x2_from_private(a_[0]),
                                 simde_poly64x2_from_private(a_[1]),
                                 simde_poly64x2_from_private(a_[2]),
